<?php
if (!defined('ABSPATH')) exit; // Exit if accessed directly

require_once 'FormStatus.php';

function formstatus_load_plugin()
{
    global $formstatus;
	
    $formstatus = new FormStatus\App\FormStatus();
}
add_action('plugins_loaded', 'formstatus_load_plugin', 100);

function formstatus_monitor_upgrader_process($upgrader_object, $options)
{
	$type = isset($options['type']) ? $options['type'] : '';
	$action = isset($options['action']) ? $options['action'] : '';
	$details = array();
	
	if ($type == 'plugin') {
		$all_plugins = get_plugins();
		$plugin = isset($options['plugin']) ? $options['plugin'] : '';
		$plugins = isset($options['plugins']) ? $options['plugins'] : array($plugin);
		
		if ($plugins) {
			foreach ($plugins as $plugin_path) {
				$plugin_data = isset($all_plugins[$plugin_path]) ? $all_plugins[$plugin_path] : array();
				$name = isset($plugin_data['Name']) ? $plugin_data['Name'] : '';
				$version = isset($plugin_data['Version']) ? $plugin_data['Version'] : '';
				
				$temp = array('type' => $type, 'name' => $name, 'version' => $version);
				
				if (count($plugins) === 1) {
					$details[$action] = $temp;
				} else {
					$details[$action][] = $temp;
				}
			}
		}
		
	} elseif ($type == 'theme') {
		$theme_data = isset($options['theme']) ? wp_get_theme($options['theme']) : false;
		$name = $theme_data ? $theme_data->get('Name') : '';
		$version = $theme_data ? $theme_data->get('Version') : '';
		
		$details[$action] = array('type' => $type, 'name' => $name, 'version' => $version);
		
	} elseif ($type == 'core') {
		$details[$action] = array('type' => $type, 'name' => get_bloginfo('name'), 'version' => get_bloginfo('version'));
	}

	formstatus_write_log(['function' => 'formstatus_monitor_upgrader_process', 'data' => $details]);
	
	formstatus_send_wp_updates($details);
}
add_action('upgrader_process_complete', 'formstatus_monitor_upgrader_process', 10, 2);

function formstatus_monitor_switch_theme($new_name, $new_theme, $old_theme)
{
	$new_theme_data = wp_get_theme($new_theme);
	$old_theme_data = wp_get_theme($old_theme);
		
	$details = array(
		'activate' => array('type' => 'theme', 'name' => $new_theme_data->get('Name'), 'version' => $new_theme_data->get('Version')),
		'deactivate' => array('type' => 'theme', 'name' => $old_theme_data->get('Name'), 'version' => $old_theme_data->get('Version'))
	);

	formstatus_write_log(['function' => 'formstatus_monitor_switch_theme', 'data' => $details]);
	
	formstatus_send_wp_updates($details);
}
add_action('switch_theme', 'formstatus_monitor_switch_theme', 10, 3);

function formstatus_monitor_active_plugins($old_value, $new_value)
{
	$all_plugins = get_plugins();
	$old_plugins = maybe_unserialize($old_value);
	$new_plugins = maybe_unserialize($new_value);
	
	$activate = array();
	if ($new_plugins) {
		foreach ($new_plugins as $new_plugin) {
			if (!in_array($new_plugin, $old_plugins)) {
				$new_plugin_data = isset($all_plugins[$new_plugin]) ? $all_plugins[$new_plugin] : array();
				$new_plugin_name = isset($new_plugin_data['Name']) ? $new_plugin_data['Name'] : '';
				$new_plugin_version = isset($new_plugin_data['Version']) ? $new_plugin_data['Version'] : '';
				
				$activate[] = array('type' => 'plugin', 'name' => $new_plugin_name, 'version' => $new_plugin_version);
			}
		}
	}
	
	$deactivate = array();
	if ($old_plugins) {
		foreach ($old_plugins as $old_plugin) {
			if (!in_array($old_plugin, $new_plugins)) {
				$old_plugin_data = isset($all_plugins[$old_plugin]) ? $all_plugins[$old_plugin] : array();
				$old_plugin_name = isset($old_plugin_data['Name']) ? $old_plugin_data['Name'] : '';
				$old_plugin_version = isset($old_plugin_data['Version']) ? $old_plugin_data['Version'] : '';
				
				$deactivate[] = array('type' => 'plugin', 'name' => $old_plugin_name, 'version' => $old_plugin_version);
			}
		}
	}
	
	$details = array();
	if ($activate) {
		$details['activate'] = (count($activate) === 1) ? $activate[0] : $activate;
	}
	if ($deactivate) {
		$details['deactivate'] = (count($deactivate) === 1) ? $deactivate[0] : $deactivate;
	}

	formstatus_write_log(['function' => 'formstatus_monitor_active_plugins', 'data' => $details]);
	
	formstatus_send_wp_updates($details);
}
add_action('update_option_active_plugins', 'formstatus_monitor_active_plugins', 10, 2);

function formstatus_send_wp_updates($details)
{
	$site_url = get_site_url();
	$domain = preg_replace('/^https?:\/\//', '', $site_url);
	$domain = rtrim($domain, '/');
	
	$date = new DateTime();
	$date->setTimezone(new DateTimeZone('UTC'));
	$datetime = $date->format('Y-m-d H:i:s');
	$timestamp = $date->getTimestamp();
	
	$default = array('domain' => $domain, 'datetime' => $datetime, 'timestamp' => $timestamp);
	$data = array_merge($default, $details);
	
    $api_url = $site_url . '/wp-update-check';
    $json_data = wp_json_encode(array(
        'data' => $data
    ));

    $response = wp_remote_post($api_url, array(
		'method' => 'POST',
		'body' => $json_data,
		'headers' => array(
			'Content-Type' => 'application/json',
			'Content-Length' => strlen($json_data),
		),
	));
	
	if (is_wp_error($response)) {
		$return = $response->get_error_message();
	} else {
		$return = 'success';
	}

	formstatus_write_log(['function' => 'formstatus_send_wp_updates', 'api_url' => $api_url, 'return' => $return]);
}

function formstatus_parse_url_parameters()
{
	$full_url = isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : '';
	if (empty($full_url)) {
		$protocol = isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http';
		$host = isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : '';
		$path = isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : '';
		if (!empty($host) && !empty($path)) {
			$full_url = $protocol . '://' . $host . $path;
		}
	}
	
	$url_params = array();
	if (!empty($full_url)) {
		$full_url = sanitize_url(wp_unslash($full_url));
		$url_components = wp_parse_url($full_url);
		if (isset($url_components['query'])) {
			parse_str($url_components['query'], $query);
			foreach ($query as $key => $value) {
				$url_params[$key] = $value;
			}
		}
    } elseif(!empty($_GET)) {
		$url_params = $_GET;
	}

	formstatus_write_log(['function' => 'formstatus_parse_url_parameters', 'return' => $url_params]);

	$formstatus_checker = isset($url_params['formstatus_checker']) ? $url_params['formstatus_checker'] : '';
    if (empty($formstatus_checker)) {
        return array();
    }

    return $url_params;
}

function formstatus_sanitize_multiarray(&$value, $key)
{
    switch ($key) {
		case 'appUrl':
			$value = sanitize_url($value);
			break;
			
		case 'postmarkEmail':
			$value = sanitize_email($value);
			break;
			
		default:
			$value = sanitize_text_field($value);
	}
}

/**
 * @param $status
 * @return bool
 */
function formstatus_send_submission_status($status)
{
    if (empty($_SESSION['formstatus'])) {
        return;
    }

	$formstatus_appurl = isset($_SESSION['formstatus']['appUrl']) ? sanitize_url($_SESSION['formstatus']['appUrl']) : '';
    $api_url = 'https://' . rtrim($formstatus_appurl, '/') . '/form/log/submit-check';
    $json_data = wp_json_encode(array(
        'data' => array_walk_recursive($_SESSION['formstatus'], 'formstatus_sanitize_multiarray'),
        'status' => $status
    ));
	
	$response = wp_remote_post($api_url, array(
		'method' => 'POST',
		'body' => $json_data,
		'headers' => array(
			'Content-Type' => 'application/json',
			'Content-Length' => strlen($json_data),
		),
	));
	
	if (is_wp_error($response)) {
		$return = $response->get_error_message();
	} else {
		$return = 'success';
	}

	formstatus_write_log(['function' => 'formstatus_send_submission_status', 'api_url' => $api_url, 'data' => $json_data, 'return' => $return]);
}

function formstatus_write_log($message)
{
	if (WP_DEBUG && WP_DEBUG_LOG) {
		error_log(print_r($message, true));
	}
}