<?php
if (!defined('ABSPATH')) exit; // Exit if accessed directly

function formstatus_contactform7_support($has_support, $tag)
{
    if ($tag === 'contact-form-7') {
        return true;
    }

    return $has_support;
}
add_filter('formstatus_support', 'formstatus_contactform7_support', 10, 2);

/**
 * Send success status if form submission succeeds.
 */
function formstatus_contactform7_success_submission($contact_form)
{
    formstatus_write_log(['function' => 'formstatus_contactform7_success_submission', 'triggered' => true]);

    formstatus_send_submission_status(true);
}
add_action('wpcf7_mail_sent', 'formstatus_contactform7_success_submission', 99999);

/**
 * Send failed status if form submission fails.
 */
function formstatus_contactform7_failed_submission($contact_form)
{
    formstatus_write_log(['function' => 'formstatus_contactform7_failed_submission', 'triggered' => true]);

    formstatus_send_submission_status(false);
}
add_action('wpcf7_mail_failed', 'formstatus_contactform7_failed_submission', 99999);

/**
 * Send failed status if form validation fails.
 */
function formstatus_contactform7_invalid_submission($result)
{
    $is_valid = $result->is_valid();

    formstatus_write_log(['function' => 'formstatus_contactform7_invalid_submission', 'is_valid' => $is_valid, 'triggered' => true]);

    if (!$is_valid) {
        formstatus_send_submission_status(false);
    }

    return $result;
}
add_filter('wpcf7_validate', 'formstatus_contactform7_invalid_submission', 99999);

/**
 * Bypass captcha for form testing.
 */
function formstatus_contactform7_bypass_captcha($result)
{
    if (empty($_SESSION['formstatus'])) {
        return $result;
    }

    $has_captcha = false;
    $invalid_fields = $result->get_invalid_fields();

	if ($invalid_fields) {
		foreach ($invalid_fields as $field_key => $field) {
			if (stripos($field_key, 'captcha') !== false) {
				$has_captcha = true;
				break;
			}
		}
	}

    formstatus_write_log(['function' => 'formstatus_contactform7_bypass_captcha', 'invalid_Fields' => $invalid_fields, 'has_captcha' => $has_captcha]);

    if (count($invalid_fields) === 1 && $has_captcha) {
        return new WPCF7_Validation();
    }

    return $result;
}
add_filter('wpcf7_validate', 'formstatus_contactform7_bypass_captcha', 99999);
function formstatus_contactform7_bypass_captchav3($is_human)
{
	if (empty($_SESSION['formstatus'])) {
        return $is_human;
    }

    formstatus_write_log(['function' => 'formstatus_contactform7_bypass_captchav3', 'default_return' => $is_human, 'return' => true]);
	
	return true;
}
add_filter('wpcf7_recaptcha_verify_response', 'formstatus_contactform7_bypass_captchav3', 99999);

/**
 * Skip spam check for form testing.
 */
function formstatus_contactform7_skip_spam_check($check)
{
    if (empty($_SESSION['formstatus'])) {
        return $check;
    }

    formstatus_write_log(['function' => 'formstatus_contactform7_skip_spam_check', 'default_return' => $check, 'return' => true]);

    return true;
}
add_filter('wpcf7_skip_spam_check', 'formstatus_contactform7_skip_spam_check', 99999);