<?php
if (!defined('ABSPATH')) exit; // Exit if accessed directly

function formstatus_gravityform_support($has_support, $tag)
{
    if ($tag === 'gravityform') {
        return true;
    }

    return $has_support;
}
add_filter('formstatus_support', 'formstatus_gravityform_support', 10, 2);

/**
 * Send status of the form submission.
 */
function formstatus_gravityform_submission($is_success)
{
    formstatus_write_log(['function' => 'formstatus_gravityform_submission', 'triggered' => true]);

    formstatus_send_submission_status($is_success);
}
add_action('gform_after_email', 'formstatus_gravityform_submission', 99999);

/**
 * Send failed status if form submission fails.
 */
function formstatus_gravityform_failed_submission($error)
{
    formstatus_write_log(['function' => 'formstatus_gravityform_failed_submission', 'triggered' => true]);

    formstatus_send_submission_status(false);
}
add_action('gform_send_email_failed', 'formstatus_gravityform_failed_submission', 99999);

/**
 * Send failed status if form validation fails.
 */
function formstatus_gravityform_invalid_submission($validation_result)
{
    $is_valid = $validation_result['is_valid'];

    formstatus_write_log(['function' => 'formstatus_gravityform_invalid_submission', 'is_valid' => $is_valid, 'triggered' => true]);

	if (!$is_valid) {
        formstatus_send_submission_status(false);
    }

    return $validation_result;
}
add_filter('gform_validation', 'formstatus_gravityform_invalid_submission', 99999);

/**
 * Remove captcha field for form testing.
 */
function formstatus_gravityform_remove_captcha_field($form)
{
    if (empty($_SESSION['formstatus'])) {
        return $form;
    }

    $fields = isset($form['fields']) ? $form['fields'] : array();

	if ($fields) {
		foreach ($fields as $key => $field) {
			if (stripos($field->type, 'captcha') !== false) {
				unset($form['fields'][$key]);
			}
		}
	}

    formstatus_write_log(['function' => 'formstatus_gravityform_remove_captcha_field', 'fields' => $fields]);

    return $form;
}
add_filter('gform_pre_render', 'formstatus_gravityform_remove_captcha_field', 99999);
add_filter('gform_pre_validation', 'formstatus_gravityform_remove_captcha_field', 99999);
add_filter('gform_pre_submission_filter', 'formstatus_gravityform_remove_captcha_field', 99999);
add_filter('gform_admin_pre_render', 'formstatus_gravityform_remove_captcha_field', 99999);

/**
 * Bypass captcha for form testing.
 */
function formstatus_gravityform_bypass_captcha($validation_result)
{
    if (empty($_SESSION['formstatus'])) {
        return $validation_result;
    }

    $errors_count = 0;
    $has_captcha_error = false;

    $form = isset($validation_result['form']) ? $validation_result['form'] : array();
    $form_fields = isset($form['fields']) ? $form['fields'] : array();

	if ($form_fields) {
		foreach ($form_fields as $field) {
            if ($field->failed_validation) {
				$errors_count += 1;

                if (stripos($field->type, 'captcha') !== false) {
				    $has_captcha_error = true;

                    foreach ( $form_fields as &$form_field ) {
                        if ( $form_field->id == $field->id ) {
                            $form_field->failed_validation = false;
                            $form_field->validation_message = '';
                        }
                    }
			    }
			}
		}
	}

    if ($errors_count === 1 && $has_captcha_error) {
        $validation_result['is_valid'] = true;
        $validation_result['form']['fields'] = $form_fields;
        $validation_result['errors'] = array();
    }

    formstatus_write_log(['function' => 'formstatus_gravityform_bypass_captcha', 'errors_count' => $errors_count, 'has_captcha_error' => $has_captcha_error, 'return' => $validation_result]);

    return $validation_result;
}
add_filter('gform_validation', 'formstatus_gravityform_bypass_captcha', 99999);
function formstatus_gravityform_bypass_captchav3($forms)
{
	if (empty($_SESSION['formstatus'])) {
        return $forms;
    }
	
	$forms['gravityformsrecaptcha']['disable-recaptchav3'] = '1';

    formstatus_write_log(['function' => 'formstatus_gravityform_bypass_captchav3', 'return' => $forms]);
	
	return $forms;
}
add_filter('gform_form_post_get_meta', 'formstatus_gravityform_bypass_captchav3', 99999);

/**
 * Skip spam check for form testing.
 */
function formstatus_gravityform_skip_spam_check($is_spam)
{
    if (empty($_SESSION['formstatus'])) {
        return $is_spam;
    }

    formstatus_write_log(['function' => 'formstatus_gravityform_skip_spam_check', 'default_return' => $is_spam, 'return' => false]);

    return false;
}
add_filter('gform_entry_is_spam', 'formstatus_gravityform_skip_spam_check', 99999);

/**
 * Delete saved entry for form testing.
 */
function formstatus_gravityform_delete_saved_entry($entry)
{
    if (empty($_SESSION['formstatus'])) {
        return;
    }

    $entry_id = isset($entry['id']) ? $entry['id'] : '';

    if (!empty($entry_id)) {
        $entry = GFAPI::get_entry($entry_id);

        if (!is_wp_error($entry)) {
            GFAPI::delete_entry($entry_id);
        }
    }

    formstatus_write_log(['function' => 'formstatus_gravityform_delete_saved_entry', 'entry' => $entry]);
}
add_action('gform_after_submission', 'formstatus_gravityform_delete_saved_entry', 99999);