<?php
if (!defined('ABSPATH')) exit; // Exit if accessed directly

function formstatus_ninjaform_support($has_support, $tag)
{
    if ($tag === 'ninja_form') {
        return true;
    }

    return $has_support;
}
add_filter('formstatus_support', 'formstatus_ninjaform_support', 10, 2);

/**
 * Form status integration on form.
 */
add_action('ninja_forms_after_form_display', function($form_id) {
	$tag = 'ninja_form';
	$attr = array('id' => $form_id);
	
	$identifier = apply_filters(
		'formstatus_form_identifier',
		$this->generate_form_identifier($tag, $attr),
		'',
		$tag,
		$attr
	);
	
	$form_has_support = apply_filters('formstatus_support', false, $tag);
	$formstatus_support = ($form_has_support) ? '1' : '';
	
	wp_register_script('formstatus-ninjaform-js-1', '', [], '', true);
	wp_enqueue_script('formstatus-ninjaform-js-1');
	wp_add_inline_script(
		'formstatus-ninjaform-js-1',
		'Backbone.Radio.channel("form").on("render:view", function(){
			jQuery(".nf-form-layout").each(function(){
				var ninjaForm = jQuery(this).find("form");
				ninjaForm.attr("data-formstatus", "1");
				ninjaForm.attr("formstatus", "' . esc_html($identifier) . '");
				ninjaForm.attr("formstatus-support", "' . esc_html($formstatus_support) . '");
			});
		});'
	);

	formstatus_write_log(['hook_function' => 'ninja_forms_after_form_display', 'tag' => $tag, 'attr' => $attr, 'identifier' => $identifier, 'has_support' => $form_has_support, 'message' => 'added using inline JS: formstatus-ninjaform-js-1']);
}, 99999);

/**
 * Send status of the form submission.
 */
function formstatus_ninjaform_submission($is_success)
{
	formstatus_write_log(['function' => 'formstatus_ninjaform_submission', 'triggered' => true]);

    formstatus_send_submission_status($is_success);

	return $is_success;
}
add_filter('ninja_forms_action_email_send', 'formstatus_ninjaform_submission', 99999);

/**
 * Send failed status if form validation fails.
 */
function formstatus_ninjaform_invalid_submission($form_data)
{
	$errors = isset($form_data['errors']) ? $form_data['errors'] : '';

	formstatus_write_log(['function' => 'formstatus_ninjaform_invalid_submission', 'errors' => $errors, 'triggered' => true]);

	if (isset($form_data['errors'])) {
		formstatus_send_submission_status(false);
	}
}
add_action('ninja_forms_before_response', 'formstatus_ninjaform_invalid_submission', 99999);
add_action('ninja_forms_after_submission', 'formstatus_ninjaform_invalid_submission', 99999);

/**
 * Remove captcha field for form testing.
 */
function formstatus_ninjaform_remove_captcha_field($field)
{
    if (empty($_SESSION['formstatus'])) {
        return $field;
    }
	
	$field['settings']['size'] = 'invisible';
	$field['value'] = '1';

	formstatus_write_log(['function' => 'formstatus_ninjaform_remove_captcha_field', 'return' => $field]);
	
	return $field;
}
add_filter('ninja_forms_localize_field_recaptcha', 'formstatus_ninjaform_remove_captcha_field', 99999);

/**
 * Validate captcha field for form testing.
 */
function formstatus_ninjaform_validate_captcha_field($field_settings)
{
	if (empty($_SESSION['formstatus'])) {
        return $field_settings;
    }
	
	if ($field_settings['type'] == 'recaptcha') {
		$field_settings['type'] = array('recaptcha');
	}

	formstatus_write_log(['function' => 'formstatus_ninjaform_validate_captcha_field', 'return' => $field_settings]);
	
	return $field_settings;
}
add_filter('ninja_forms_pre_validate_field_settings', 'formstatus_ninjaform_validate_captcha_field', 99999);

/**
 * Bypass captcha for form testing.
 */ 
function formstatus_ninjaform_bypass_captcha($is_human)
{
	if (empty($_SESSION['formstatus'])) {
        return $is_human;
    }

	formstatus_write_log(['function' => 'formstatus_ninjaform_bypass_captcha', 'default_return' => $is_human, 'return' => true]);
	
	return true;
}
add_filter('ninja_forms_action_recaptcha__verify_response', 'formstatus_ninjaform_bypass_captcha', 99999);

/**
 * Skip spam check for form testing.
 */
function formstatus_ninjaform_skip_spam_check($form_id)
{
    if (empty($_SESSION['formstatus'])) {
        return;
    }

	wp_register_script('formstatus-ninjaform-js-2', '', [], '', true);
	wp_enqueue_script('formstatus-ninjaform-js-2');
	wp_add_inline_script(
		'formstatus-ninjaform-js-2',
		'Backbone.Radio.channel("form").on("render:view", function(){
			jQuery(".nf-form-hp").each(function(){
				jQuery(this).remove();
			});
		});'
	);

	formstatus_write_log(['function' => 'formstatus_ninjaform_skip_spam_check', 'message' => 'removed using inline JS: formstatus-ninjaform-js-2']);
}
add_action('ninja_forms_after_form_display', 'formstatus_ninjaform_skip_spam_check', 99999);

/**
 * Disable saving of entry for form testing.
 */
function formstatus_ninjaform_disable_saving_entry($save)
{
    if (empty($_SESSION['formstatus'])) {
        return $save;
    }

	formstatus_write_log(['function' => 'formstatus_ninjaform_disable_saving_entry', 'default_return' => $save, 'return' => false]);

    return false;
}
add_filter('ninja_forms_save_submission', 'formstatus_ninjaform_disable_saving_entry', 99999);